
# coding: utf-8

## Classifying whether a mushroom is poisonous

# In[2]:

import scipy.sparse as sp
import numpy as np


# In[3]:

train = open('data/agaricus.txt.train').readlines()


# In[4]:

train[0:3]


# ## Write a function to parse LibSVM File

# In[5]:

# read svm file into csc matrix
def readlibsvm(fname):
    row = [] 
    col = []
    data = []
    label = []
    icnt=0
    for l in open(fname):
        arr = l.split()
        label.append(int(arr[0]))
        for it in arr[1:]:
            k,v = it.split(':')
            col.append(int(k))
            data.append(float(v))
            row.append(icnt)
        icnt += 1
    return np.matrix(np.array(label)).T, sp.csc_matrix((data,(row,col)))
ytrain, Xtrain = readlibsvm('data/agaricus.txt.train')
ytest, Xtest = readlibsvm('data/agaricus.txt.test')


# # Fit a Linear Regression using Ridge Model
# Let us set $w_0 = 0.5$
# 
# $\hat{w} = argmin_w  \frac{1}{2}\|X w + w_0- y \|^2 + \frac{1}{2}\lambda \|w\|^2$
# 
# $\hat{w} = (X^T X + \lambda I)^{-1} X^T (y - w_0) $ 
# 

# In[6]:

def least_square(X, y):
    n,d = X.shape
    r = 1
    w0 = 0.5
    Xcov = X.T * X + np.diag(np.ones(d)) * r
    what = np.linalg.inv(Xcov) * (X.T*(y-w0))
    return what
what = least_square(Xtrain, ytrain)


# ## Predict and evaluate result

# In[7]:

def predict(X,w):
    return X * w + 0.5 
def evaluate(ypred, y):
    yhard = ypred > 0.5
    return np.sum(yhard != y)/float(len(y))
n,d = Xtrain.shape
wrand = np.matrix(np.random.randn(d)).T
print 'error with random weight = %g' %evaluate(predict(Xtrain,wrand), ytrain)
print 'training error = %g' %evaluate(predict(Xtrain,what), ytrain)
print 'testing error = %g' %evaluate(predict(Xtest,what), ytest)


# In[2]:




# ## Inspect and interpret the result

# In[8]:

open('data/featmap.txt').readlines()[:5]
fmap ={}
for l in open('data/featmap.txt'):
    arr = l.split()
    fmap[len(fmap)] = arr[1]


# In[10]:

sorted( [(i, what[i,0], fmap[i]) for  i in range(d)], key = lambda x:-abs(x[1]))


# In[ ]:



