
# coding: utf-8

# # Numpy Walkthrough

# In[2]:

import numpy as np


# In[3]:

# properties of ndarry
A = np.arange(15)
print A
A = A.reshape(3,5)
print A
print A.shape
print A.ndim
print A.dtype
print A.size


# In[4]:

# cell for typing 
mylist = [1,2,3]
np.array(mylist)


# In[5]:

# construct array from list
mylist = [2,3,4,1.2]
A = np.array(mylist)
1 / A


# In[6]:

# operations
A + 1
A / 2


# In[7]:

# some useful constructions
np.zeros((3,3))


# ## Numpy Matrix and Array
# * usually, numpy takes ndarray tensor syntax
# * numpy.matrix takes matrix syntax
# * try not to mix the usage..

# In[8]:

# elementwise operations
A = np.array([[1,1], [0,1]])
B = np.array([[2,0], [3,4]])
print A
print B
A + B
A * B
print np.dot(A.T,B)


# In[9]:

# matrix operations
print np.dot(A, B)
print A * B
mA = np.matrix(A)
mB = np.matrix(B)
mA * mB


# In[11]:

# cell to type 


# In[12]:

#indexing
A = np.arange(15).reshape(3,5)
# first row
print A[1]
# column 2
print A[:,2]
# row 1,2 with all columns
print A[1:3, :]


# In[13]:

# copy
A = np.arange(15)
A


# In[14]:

# advanced indexing
A[A > 1] =10
A[A % 2 == 0]


# In[15]:

# cell to type


# ## Sparse Matrix: scipy.sparse.csc_matrix
# Although ndarray is good in numpy, scipy.sparse takes numpy.matrix syntax

# In[16]:

import numpy as np
import scipy.sparse as sp


# In[17]:

A = np.arange(12).reshape((3,4))
A[1,1] = 0
S = sp.csc_matrix(A)
w = np.ones(4).reshape(4,1)
print A
print w


# ## Matrix multiplication 

# In[18]:

# rememeber csc_matrix is matrix instead ndarray 
print np.dot(A, w)
print S * np.matrix(w)


# ## Operations over columns
# $\sum_i S_{ik} y_i = S_{:,k}^T y $

# In[19]:

y = np.arange(3)
k = 1
(S[:,k].T * y)[0]


# ## Looking inside the data structure
# * The data structure is called **Compressed Sparse Column matrix**

# In[20]:

A


# In[21]:

S.indptr


# In[22]:

S.indices


# In[23]:

S.data


# In[24]:

start, end = S.indptr[1], S.indptr[2]
S.indices[start:end]


# In[25]:

S.data[start:end]


# $\sum_{X_{jk}\neq 0} X_{jk} y_j$ 

# In[26]:

np.sum(y[S.indices[start:end]] * S.data[start:end])

